@@ -19,7 +19,7 @@ use paste::paste;
 
 // todo: Low power timer enabling etc. eg on L4, RCC_APB1ENR1.LPTIM1EN
 
-#[derive(Clone, Copy)]
+#[derive(Clone, Copy, Debug)]
 /// Used for when attempting to set a timer period that is out of range.
 pub struct ValueError {}
 
@@ -662,17 +662,37 @@ macro_rules! pwm_features {
 // We only implement `pwm_features` for general purpose timers. Perhaps we should implement
 // for advanced-control timers too.
 
-#[cfg(not(any(feature = "f373", feature = "f4")))]
+#[cfg(not(any(feature = "f373")))]
 hal!(TIM1, tim1, 2);
-
 #[cfg(not(any(
-    feature = "f301",
-    feature = "f552",
-    feature = "l4x3",
-    feature = "l4x1",
-    feature = "l552",
-    feature = "f410",
+    feature = "f373",
+    feature = "f4",
+    feature = "l4",
+    feature = "l5",
+    feature = "g0"
 )))]
+pwm_features!(TIM1, u16);
+
+#[cfg(any(feature = "g0"))]
+pwm_features!(TIM1, u32);
+
+cfg_if! {
+    if #[cfg(not(any(
+        feature = "f410",
+        feature = "g070",
+    )))] {
+        hal!(TIM2, tim2, 1);
+    }
+}
+
+// todo: G4 has tim2, and it's 32-bit, but there may be a PAC error here; pac expects arr to be 16 bit.
+#[cfg(feature = "g4")]
+pwm_features!(TIM2, u16);
+
+#[cfg(not(any(feature = "l5", feature = "g070", feature = "g4", feature = "f410")))]
+pwm_features!(TIM2, u32);
+
+#[cfg(not(any(feature = "f301", feature = "l4x1", feature = "l4x3", feature = "f410",)))]
 hal!(TIM3, tim3, 1);
 
 #[cfg(not(any(
@@ -691,38 +711,70 @@ cfg_if! {
     if #[cfg(not(any(
         feature = "f301",
         feature = "f3x4",
+        feature = "f410",
         feature = "l4x1",
         feature = "l4x2",
         feature = "l4x3",
         feature = "l552",
-        feature = "f4",
         feature = "g0",
     )))] {
         hal!(TIM4, tim4, 1);
-        hal!(TIM17, tim17, 2);
     }
 }
 
-#[cfg(not(any(
-    feature = "f3x4",
-    feature = "l4x1",
-    feature = "l4x2",
-    feature = "l4x3",
-    feature = "l5",
-    feature = "f410",
-    feature = "g0",
-)))]
-pwm_features!(TIM4, u16);
+cfg_if! {
+    if #[cfg(not(any(
+        feature = "f301",
+        feature = "f3x4",
+        feature = "f410",
+        feature = "l4x1",
+        feature = "l4x2",
+        feature = "l4x3",
+        feature = "l5",
+        feature = "g0",
+    )))] {
+        pwm_features!(TIM4, u16);
+    }
+}
 
-#[cfg(any(
-    feature = "f373",
-    feature = "l4x5",
-    feature = "l4x6",
-    feature = "l562",
-    feature = "h7",
-    feature = "f4",
-))]
-hal!(TIM5, tim5, 1);
+cfg_if! {
+    if #[cfg(any(
+       feature = "f373",
+       feature = "l4x5",
+       feature = "l4x6",
+       feature = "l562",
+       feature = "h7",
+       all(feature = "f4", not(feature = "f410")),
+   ))] {
+        hal!(TIM5, tim5, 1);
+   }
+}
+
+cfg_if! {
+    if #[cfg(any(
+       feature = "f373",
+       feature = "l4x5",
+       feature = "l4x6",
+       feature = "h7",
+       all(feature = "f4", not(feature = "f410")),
+   ))] {
+        pwm_features!(TIM5, u32);
+   }
+}
+
+cfg_if! {
+    if #[cfg(not(any(
+        feature = "f401",
+        feature = "f410",
+        feature = "f411",
+        feature = "g031",
+        feature = "g041",
+        feature = "g070",
+        feature = "g030"
+    )))] {
+        hal!(TIM6, tim6, 1);
+    }
+}
 
 #[cfg(not(any(
     feature = "f301",
@@ -736,23 +788,14 @@ hal!(TIM5, tim5, 1);
 )))]
 hal!(TIM7, tim7, 1);
 
-cfg_if! {
-    if #[cfg(not(any(
-        feature = "f401",
-        feature = "f410",
-        feature = "f411",
-        feature = "g031",
-        feature = "g041",
-        feature = "g070",
-        feature = "g030"
-    )))] {
-        hal!(TIM2, tim2, 1);
-        hal!(TIM6, tim6, 1);
-    }
-}
-
-#[cfg(not(feature = "f4"))]
-hal!(TIM16, tim16, 2);
+#[cfg(any(
+    feature = "f303",
+    feature = "l4x5",
+    feature = "l4x6",
+    feature = "l562",
+    feature = "g4"
+))]
+hal!(TIM8, tim8, 2);
 
 // Todo: the L5 PAC has an address error on TIM15 - remove it until solved.
 #[cfg(not(any(
@@ -765,32 +808,24 @@ hal!(TIM16, tim16, 2);
 )))]
 hal!(TIM15, tim15, 2);
 
-// todo: G4 has tim2, and it's 32-bit, but there may be a PAC error here; pac expects arr to be 16 bit.
-#[cfg(not(any(
-    feature = "l4x1",
-    feature = "l5",
-    feature = "g4",
-    feature = "f410",
-    feature = "g070"
-)))]
-pwm_features!(TIM2, u32);
+#[cfg(not(feature = "f4"))]
+hal!(TIM16, tim16, 2);
 
-#[cfg(any(
-    feature = "f303",
-    feature = "l4x5",
-    feature = "l4x6",
-    feature = "l562",
-    feature = "g4"
-))]
-hal!(TIM8, tim8, 2);
+cfg_if! {
+    if #[cfg(not(any(
+        feature = "l4x1",
+        feature = "l4x2",
+        feature = "l4x3",
+        feature = "f4",
+    )))] {
+        hal!(TIM17, tim17, 2);
+    }
+}
 
 // { todo: tim18
 //     TIM18: (tim18, apb2, enr, rstr),
 // },
 
-#[cfg(any(feature = "f303"))]
-hal!(TIM20, tim20, 2);
-
 cfg_if! {
     if #[cfg(any(feature = "f373"))] {
         hal!(TIM12, tim12, 1);
@@ -800,4 +835,5 @@ cfg_if! {
     }
 }
 
-// todo: Figure out which timers G4 has, and add A/R.
+#[cfg(any(feature = "f303"))]
+hal!(TIM20, tim20, 2);
