@@ -43,14 +43,3 @@ pub trait ClockCfg {
     /// // todo Separate USB validation? back to `Validation enum`, or keep it simple?
     fn validate_speeds(&self) -> ClocksValid;
 }
-
-/// Trait representing a single-channel digital-to-analog converter (DAC).
-pub trait SingleChannelDac<Word> {
-    /// Error type returned by DAC methods
-    type Error;
-
-    /// Output a constant signal, given a bit word.
-    fn try_set_value(&mut self, value: Word) -> Result<(), Self::Error>;
-
-    // todo: Enable and disable? set voltage?
-}
